return [==[
// Generated by CoffeeScript 1.9.2
(function() {
  var debounce,
    slice = [].slice,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  window.Lapis || (window.Lapis = {});

  debounce = function(fn, time) {
    var t;
    t = null;
    return function() {
      var args;
      args = 1 <= arguments.length ? slice.call(arguments, 0) : [];
      if (t) {
        clearTimeout(t);
      }
      return t = setTimeout(function() {
        t = null;
        return fn.apply(null, args);
      }, time);
    };
  };

  Lapis.Editor = (function() {
    Editor.prototype.session_name = "lapis-console-session";

    Editor.prototype.run_code = function(code, fn) {
      var opts;
      if (this.last_line_handle) {
        this.editor.removeLineClass(this.last_line_handle, "background", "has_error");
      }
      opts = $.param({
        lang: "moonscript"
      });
      this.set_status("loading", "Loading...");
      return $.post("?" + opts, {
        code: code
      }, (function(_this) {
        return function(res) {
          var line_no, m;
          if (res.error) {
            _this.set_status("error", res.error);
            if (m = res.error.match(/\[(\d+)\]/)) {
              line_no = parseInt(m[1], 10);
              return _this.last_line_handle = _this.editor.addLineClass(line_no - 1, "background", "has_error");
            }
          } else {
            _this.set_status("ready", "Ready");
            return typeof fn === "function" ? fn(res) : void 0;
          }
        };
      })(this));
    };

    Editor.prototype.set_status = function(name, msg) {
      return (this._status || (this._status = this.el.find(".status"))).removeClass("error ready loading").addClass(name).text(msg);
    };

    Editor.prototype.expand_object = function(el) {
      var i, k, len, ref, tuples, v;
      tuples = el.data("tuples");
      el.empty().removeClass("expandable").addClass("expanded");
      el.append("<div class='closable'>{</div>");
      for (i = 0, len = tuples.length; i < len; i++) {
        ref = tuples[i], k = ref[0], v = ref[1];
        $('<div class="tuple"></div>').append(this.render_value(k).addClass("key")).append(this.render_value(v)).appendTo(el);
      }
      return el.append("<div class='closable'>}</div>");
    };

    Editor.prototype.close_object = function(el) {
      return el.text("{ ... }").removeClass("expanded").addClass("expandable");
    };

    Editor.prototype.render_value = function(val) {
      var content, has_content, type, val_el;
      val_el = $('<pre class="value"></pre>');
      type = val[0], content = val[1];
      if (type === "table") {
        has_content = content.length > 0;
        val_el.text("{ " + (has_content && "..." || "") + " }").addClass("object expandable").toggleClass("expandable", has_content).data("tuples", content);
      } else {
        val_el.addClass(type).text(content);
      }
      val_el.attr("title", type);
      return val_el;
    };

    Editor.prototype.render_result = function(res) {
      var i, j, l, len, len1, len2, line, line_el, lines_el, q, queries_el, ref, ref1, row, value;
      row = $("<div class=\"result\">\n  <div class=\"lines\"></div>\n  <div class=\"queries\"></div>\n</div>");
      lines_el = row.find(".lines");
      queries_el = row.find(".queries");
      ref = res.lines;
      for (i = 0, len = ref.length; i < len; i++) {
        line = ref[i];
        line_el = $('<div class="line"></div>');
        for (j = 0, len1 = line.length; j < len1; j++) {
          value = line[j];
          this.render_value(value).appendTo(line_el);
        }
        line_el.appendTo(lines_el);
      }
      ref1 = res.queries;
      for (l = 0, len2 = ref1.length; l < len2; l++) {
        q = ref1[l];
        $("<div class='query'></div>").text(q).appendTo(queries_el);
      }
      if (!res.queries[0]) {
        queries_el.remove();
      }
      if (!res.queries[0] && !res.lines[0]) {
        row.addClass("no_output").text("No output");
      }
      return this.log.prepend(row);
    };

    Editor.prototype.save_session = function() {
      console.log("saved_session");
      if (!window.sessionStorage) {
        return;
      }
      return window.sessionStorage.setItem(this.session_name, this.editor.getValue());
    };

    Editor.prototype.load_session = function() {
      var prev_value;
      if (!window.sessionStorage) {
        return;
      }
      prev_value = window.sessionStorage.getItem(this.session_name);
      if (prev_value) {
        return this.editor.setValue(prev_value);
      }
    };

    function Editor(el) {
      this.load_session = bind(this.load_session, this);
      this.save_session = bind(this.save_session, this);
      this.render_result = bind(this.render_result, this);
      this.render_value = bind(this.render_value, this);
      this.close_object = bind(this.close_object, this);
      this.expand_object = bind(this.expand_object, this);
      var clear_handler, run_handler;
      this.el = $(el);
      this.log = this.el.find(".log");
      this.textarea = this.el.find("textarea");
      this.editor = CodeMirror.fromTextArea(this.textarea[0], {
        mode: "moonscript",
        lineNumbers: true,
        tabSize: 2,
        theme: "moon",
        viewportMargin: Infinity
      });
      this.editor.on("change", debounce((function(_this) {
        return function() {
          return _this.save_session();
        };
      })(this), 500));
      run_handler = (function(_this) {
        return function() {
          _this.run_code(_this.editor.getValue(), function(res) {
            return _this.render_result(res);
          });
          return false;
        };
      })(this);
      clear_handler = (function(_this) {
        return function() {
          _this.editor.setValue("");
          delete _this.last_line_handle;
          return false;
        };
      })(this);
      this.editor.addKeyMap({
        "Ctrl-Enter": run_handler,
        "Ctrl-K": clear_handler
      });
      this.el.on("click", ".run_btn", run_handler);
      this.el.on("click", ".clear_btn", clear_handler);
      $((function(_this) {
        return function() {
          return _this.editor.focus();
        };
      })(this));
      this.log.on("click", ".expandable", (function(_this) {
        return function(e) {
          return _this.expand_object($(e.currentTarget));
        };
      })(this));
      this.log.on("click", ".closable", (function(_this) {
        return function(e) {
          return _this.close_object($(e.currentTarget).closest(".object"));
        };
      })(this));
      this.load_session();
    }

    return Editor;

  })();

}).call(this);
]==]
